//---------------------------------------------------------------------------

#ifndef AcotGridH
#define AcotGridH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <Grids.hpp>
//---------------------------------------------------------------------------
class TGridColumnAgs;
class TAcotGrid;
enum TInputDataType { dtString, dtInteger };
enum TGridStyle { gsNormal, gsStriped };

#include "ObjList.h"

//---------------------------------------------------------------------------
class TSelRangeList;
class TDocoCmd;

enum TDocoCmdType { dcSubst, dcInsert, dcDelete };

typedef CObjList<TDocoCmd> TDocoCmdList;
typedef CObjList<TDocoCmdList> TUndoList;

//---------------------------------------------------------------------------
class TInplaceEditAg : public Grids::TInplaceEdit
{
	friend class TAcotGrid;
private:
	bool FMultiLine;
	bool FWordWrap;
	void __fastcall SetWordWrap(bool Value);
	
protected:
	virtual void __fastcall CreateParams(Controls::TCreateParams &Params);
	DYNAMIC void __fastcall KeyDown(System::Word &Key, Classes::TShiftState Shift);
	DYNAMIC void __fastcall KeyUp(System::Word &Key, Classes::TShiftState Shift);
	DYNAMIC void __fastcall KeyPress(System::WideChar &Key);
	
__published:
	__property ImeMode = {default=3};
	__property bool MultiLine = {read=FMultiLine, write=FMultiLine, nodefault};
	__property bool WordWrap = {read=FWordWrap, write=SetWordWrap, nodefault};
public:
	/* TInplaceEdit.Create */ inline __fastcall virtual TInplaceEditAg(Classes::TComponent* AOwner) : Grids::TInplaceEdit(AOwner) { }
	
public:
	/* TWinControl.CreateParented */ inline __fastcall TInplaceEditAg(HWND ParentWindow) : Grids::TInplaceEdit(ParentWindow) { }
	/* TWinControl.Destroy */ inline __fastcall virtual ~TInplaceEditAg(void) { }
	
};
//---------------------------------------------------------------------------
class TGridColumnAg : public Classes::TCollectionItem
{
private:
	Classes::TAlignment FAlignment;
	bool FCanEdit;
	Graphics::TColor FColor;
	System::UnicodeString FColumnName;
	TInputDataType FDataType;
	System::UnicodeString FDisplayMask;
	Graphics::TFont* FFixCellFont;
	Graphics::TFont* FFont;
	Controls::TImeMode FImeMode;
	bool FMultiLine;
	bool FWordWrap;
	void __fastcall SetColor(Graphics::TColor Value);
	void __fastcall SetFixCellFont(Graphics::TFont* Value);
	void __fastcall SetFont(Graphics::TFont* Value);
	void __fastcall SetWordWrap(bool Value);
	
public:
	__fastcall virtual TGridColumnAg(Classes::TCollection* Collection);
	__fastcall virtual ~TGridColumnAg(void);
	virtual void __fastcall Assign(Classes::TPersistent* Value);
	void __fastcall SetAlignment(Classes::TAlignment Value);
	void __fastcall SetColumnName(System::UnicodeString Value);
	
__published:
	__property Classes::TAlignment Alignment = {read=FAlignment, write=SetAlignment, nodefault};
	__property bool CanEdit = {read=FCanEdit, write=FCanEdit, nodefault};
	__property System::UnicodeString Caption = {read=FColumnName, write=SetColumnName};
	__property Graphics::TColor Color = {read=FColor, write=SetColor, nodefault};
	__property TInputDataType DataType = {read=FDataType, write=FDataType, nodefault};
	__property System::UnicodeString DisplayMask = {read=FDisplayMask, write=FDisplayMask};
	__property Graphics::TFont* Font = {read=FFont, write=SetFont};
	__property Graphics::TFont* FixCellFont = {read=FFixCellFont, write=SetFixCellFont};
	__property Controls::TImeMode ImeMode = {read=FImeMode, write=FImeMode, nodefault};
	__property bool WordWrap = {read=FWordWrap, write=SetWordWrap, default=0};
	__property bool MultiLine = {read=FMultiLine, write=FMultiLine, default=0};
};
//---------------------------------------------------------------------------
class TGridColumnAgs : public Classes::TCollection
{
	friend class TGridColumnAg;
private:
	bool FCanSizing;
	TAcotGrid* FAcotGrid;
	void __fastcall SetItems(int Index, TGridColumnAg* Value);
	TGridColumnAg* __fastcall GetItems(int Index);
	
protected:
	virtual void __fastcall Update(Classes::TCollectionItem* Item);
	DYNAMIC Classes::TPersistent* __fastcall GetOwner(void);
	
public:
	__fastcall TGridColumnAgs(TAcotGrid* AcotGrid);
	HIDESBASE TGridColumnAg* __fastcall Add(void);
	__property TGridColumnAg* Items[int Index] = {read=GetItems, write=SetItems};
	__property bool CanSizing = {read=FCanSizing, write=FCanSizing, nodefault};
public:
	/* TCollection.Destroy */ inline __fastcall virtual ~TGridColumnAgs(void) { }
	
};
//---------------------------------------------------------------------------
class PACKAGE TAcotGrid : public TStringGrid
{
	friend class TInplaceEditAg;
private:
	TGridColumnAgs* FGridColumnAgs;
	bool FlagSelRange;
	bool FlagCellEdit;
	void __fastcall SetCellEdit(bool value);
	bool __fastcall GetCellEdit();
	TSelRangeList *SelRangeList;
	TGridCoord CurrentPt;
	TGridCoord AnchorPt;
	TPopupMenu *GridPopup;
	TPopupMenu *ColPopup;
	TPopupMenu *RowPopup;
	TGridCoord PopupCoord;
	TCHAR CsvSeparator;
	TUndoList *UndoList;
	int UndoIndex;
	int UndoCountMark; // ύX̗LǗ邽߂ UndoList->Count L^
    int DownX, DownY;
    int MoveX, MoveY;

	UnicodeString __fastcall GetCells(int ACol, int ARow);
	void __fastcall SetCells(int ACol, int ARow, UnicodeString val);
	int __fastcall GetRowCount();
	void __fastcall SetRowCount(int value);
	bool __fastcall IsSelCtrlKey(Word Key, Classes::TShiftState Shift);
	TGridState __fastcall GetGridState(int X, int Y);

	void __fastcall SelCopy(bool fclear);
	void __fastcall SelPaste();
	void __fastcall SelClear();
	void __fastcall InsCol(bool fAppend);
	void __fastcall InsertCol(int col);
	void __fastcall DelCol();
	void __fastcall DeleteCol(int delcol);
	void __fastcall InsRow(bool fAppend);
	void __fastcall InsertRow(int row);
	void __fastcall DelRow();
	void __fastcall DeleteRow(int delrow);
	void __fastcall Undo();
	void __fastcall Redo();
	void __fastcall UndoAddEditText(int col, int row, UnicodeString sold, UnicodeString snew);
	void __fastcall UndoAdd(TDocoCmdType cmd, int col, int row, UnicodeString sold, UnicodeString snew);
	void __fastcall UndoAdd(TDocoCmdList *cmdlist);

	void __fastcall ResetSelRange(Classes::TShiftState Shift);
	void __fastcall GridPopupClick(System::TObject *Sender);
	void __fastcall ContextPopup(System::TObject* Sender,
							const Types::TPoint &MousePos, bool &Handled);

protected:
	TDocoCmdList *DocoCmdList; // MOD_20081117

	DYNAMIC void __fastcall MouseDown(Controls::TMouseButton Button, Classes::TShiftState Shift, int X,int Y);
	DYNAMIC void __fastcall MouseUp(Controls::TMouseButton Button, Classes::TShiftState Shift, int X, int Y);
	DYNAMIC void __fastcall MouseMove(Classes::TShiftState Shift, int X, int Y);
	DYNAMIC void __fastcall DblClick(void);
	DYNAMIC void __fastcall KeyDown(Word &Key, Classes::TShiftState Shift);
	DYNAMIC void __fastcall KeyUp(Word &Key, Classes::TShiftState Shift);
	virtual void __fastcall DrawCell(int ACol, int ARow, const TRect &ARect, TGridDrawState AState);
	DYNAMIC void __fastcall SizeChanged(int OldColCount, int OldRowCount);
	DYNAMIC bool __fastcall CanEditModify(void);
	DYNAMIC void __fastcall SetEditText(int ACol, int ARow, const UnicodeString Value);
	virtual void __fastcall WndProc(Messages::TMessage &Message);
	virtual Grids::TInplaceEdit* __fastcall CreateEditor(void);
	DYNAMIC UnicodeString __fastcall GetEditText(int ACol, int ARow);
	void __fastcall SetSelection(const TGridRect &Value);
	TGridRect __fastcall GetSelection(void);

	UnicodeString __fastcall EscapeText(UnicodeString str);
	UnicodeString __fastcall XsvText(TStrings *list, TCHAR separator);
	int __fastcall GetCsvColumn(TCHAR *str, UnicodeString &rtn, TCHAR sepchar);
	TCHAR __fastcall JudgeSepChar(const UnicodeString &str);

	static UnicodeString __fastcall ColNumber(int n);
public:
	__fastcall TAcotGrid(TComponent* Owner);
	__fastcall ~TAcotGrid();

	bool RepeatPaste;
	TGetEditEvent OnGetCopyText;
	TSetEditEvent OnSetPasteText;

	void __fastcall ClearSelRange();
	void __fastcall ClearUndo();
	void __fastcall SetParms(TStringGrid *sgrid);
	void __fastcall SetCsvSeparator(TCHAR sepchar);
	void __fastcall BeginSubst();
	void __fastcall EndSubst();
	void __fastcall SubstCell(int ACol, int ARow, UnicodeString val);
	void __fastcall AdjustRowHeight(int ARow);
	int __fastcall GetModCount();
	void __fastcall ResetModCount();

	__property UnicodeString Cells[ACol][ARow] = {read=GetCells, write=SetCells};
	__property bool CellEdit = {read=GetCellEdit, write=SetCellEdit};
	__property Selection = {read=GetSelection, write=SetSelection};
__published:
	__property TGridColumnAgs* GridColumnAgs = {read=FGridColumnAgs};
	__property int RowCount = {read=GetRowCount, write=SetRowCount, default=5};
};
//---------------------------------------------------------------------------
#endif
